window.Gleam.initSearch([{"doc":"based","title":"based","content":"# based\n\n[![Package Version](https://img.shields.io/hexpm/v/based)](https://hex.pm/packages/based)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/based/)\n\nThis package provides a unified interface for interacting SQL databases. With\n`based`, gleam code can remain mostly database-agnostic. SQL queries will need\nto be written for your chosen database (postgres, sqlite, etc), but execution\nof those queries isn&#39;t tied to your chosen database.\n\nPackages for different database backends can conform to the type and function\ndefinitions in this package. This can allow developers to quickly get started\nwriting gleam programs using a sqlite backend, but change to postgres when\nneeded without a large refactor.\n\n## Example packages that can be used with `based`\n\n- [`based_pg`](https://github.com/stndrs/based_pg)\n- [`based_sqlite`](https://github.com/stndrs/based_sqlite)\n\n```sh\ngleam add based\n```\n\n```gleam\nimport based\nimport dynamic\nimport based_pg\n// import based_sqlite\n\nconst sql = &quot;SELECT name FROM users WHERE id=$1;&quot;\n\npub type User {\n  User(name: String)\n}\n\npub fn main() {\n  let config = load_config()\n\n  use db &lt;- based.register(based_pg.adapter(config))\n  // use db &lt;- based.register(based_sqlite.adapter(config))\n\n  let decoder = dynamic.decode1(User, dynamic.element(0, dynamic.string))\n\n  // Swapping out the backend doesn&#39;t require rewriting the existing queries\n  based.new_query(sql)\n  |&gt; based.with_values([based.int(1)])\n  |&gt; based.one(db, decoder)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/based&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"based","title":"BasedAdapter","content":"pub type BasedAdapter(conf, conn, t) {\n  BasedAdapter(\n    with_connection: WithConnection(conf, conn, t),\n    conf: conf,\n    service: Service(conn),\n  )\n}\n\nBasedAdapter(\n  with_connection: WithConnection(conf, conn, t),\n  conf: conf,\n  service: Service(conn),\n)\n\n\nSynonyms:\nbased.BasedAdapter\nbased BasedAdapter","url":"based.html#BasedAdapter"},{"doc":"based","title":"BasedError","content":"pub type BasedError {\n  BasedError(code: String, name: String, message: String)\n}\n\nBasedError(code: String, name: String, message: String)\n\n\nSynonyms:\nbased.BasedError\nbased BasedError","url":"based.html#BasedError"},{"doc":"based","title":"DB","content":"pub opaque type DB\n\n\nSynonyms:\nbased.DB\nbased DB","url":"based.html#DB"},{"doc":"based","title":"Message","content":"pub type Message {\n  Execute(\n    reply_with: Subject(Result(List(Dynamic), BasedError)),\n    query: Query,\n  )\n  Shutdown\n}\n\nExecute(\n  reply_with: Subject(Result(List(Dynamic), BasedError)),\n  query: Query,\n)\n\n\nShutdown\n\n\nSynonyms:\nbased.Message\nbased Message","url":"based.html#Message"},{"doc":"based","title":"Query","content":"pub type Query {\n  Query(sql: String, values: List(Value))\n}\n\nQuery(sql: String, values: List(Value))\n\n\nSynonyms:\nbased.Query\nbased Query","url":"based.html#Query"},{"doc":"based","title":"Returned","content":"pub type Returned(a) {\n  Returned(count: Int, rows: List(a))\n}\n\nReturned(count: Int, rows: List(a))\n\n\nSynonyms:\nbased.Returned\nbased Returned","url":"based.html#Returned"},{"doc":"based","title":"Service","content":"pub type Service(conn) =\n  fn(Query, conn) -&gt; Result(List(Dynamic), BasedError)\n\n\nSynonyms:\nbased.Service\nbased Service","url":"based.html#Service"},{"doc":"based","title":"Value","content":"pub type Value {\n  String(String)\n  Int(Int)\n  Float(Float)\n  Bool(Bool)\n  Null\n}\n Callers will interact with these Value types when building queries. Their chosen\n backend is responsible for converting these Value types to the appropriate type.\nString(String)\n\n\nInt(Int)\n\n\nFloat(Float)\n\n\nBool(Bool)\n\n\nNull\n\n\nSynonyms:\nbased.Value\nbased Value","url":"based.html#Value"},{"doc":"based","title":"WithConnection","content":"pub type WithConnection(conf, conn, t) =\n  fn(conf, fn(conn) -&gt; t) -&gt; t\n Defines a valid `with_connection` function\n\nSynonyms:\nbased.WithConnection\nbased WithConnection","url":"based.html#WithConnection"},{"doc":"based","title":"all","content":"pub fn all(\n  query: Query,\n  db: DB,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Returned(a), BasedError)\n Applies the provided `Decoder` to all rows returned by the Query.\nSynonyms:\nbased.all\nbased all","url":"based.html#all"},{"doc":"based","title":"bool","content":"pub fn bool(value: Bool) -&gt; Value\n Converts a bool to a `Value` type\nSynonyms:\nbased.bool\nbased bool","url":"based.html#bool"},{"doc":"based","title":"decode","content":"pub fn decode(\n  rows: List(Dynamic),\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Returned(a), BasedError)\n Decodes a list of Dynamic values with the provided `Decoder`.\nSynonyms:\nbased.decode\nbased decode","url":"based.html#decode"},{"doc":"based","title":"execute","content":"pub fn execute(\n  query: Query,\n  db: DB,\n) -&gt; Result(List(Dynamic), BasedError)\n Performs a query and returns a list of Dynamic values.\nSynonyms:\nbased.execute\nbased execute","url":"based.html#execute"},{"doc":"based","title":"float","content":"pub fn float(value: Float) -&gt; Value\n Converts a float to a `Value` type\nSynonyms:\nbased.float\nbased float","url":"based.html#float"},{"doc":"based","title":"int","content":"pub fn int(value: Int) -&gt; Value\n Converts an int to a `Value` type\nSynonyms:\nbased.int\nbased int","url":"based.html#int"},{"doc":"based","title":"new_query","content":"pub fn new_query(sql: String) -&gt; Query\n Returns a Query record with the provided SQL string and an empty list of values.\n This function can be used on its own for queries that don&#39;t require values. Its\n return value may also be piped into `with_values` to be given the appropriate\n list of values required for the query.\nSynonyms:\nbased.new_query\nbased new_query","url":"based.html#new_query"},{"doc":"based","title":"null","content":"pub fn null() -&gt; Value\n Returns a Null Value type\nSynonyms:\nbased.null\nbased null","url":"based.html#null"},{"doc":"based","title":"one","content":"pub fn one(\n  query: Query,\n  db: DB,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(a, BasedError)\n Returns the first row returned by the query after being decoded by the provided\n `Decoder`. Useful for queries where only one row should be returned. If more rows\n are returned by the query, only the first row will be decoded and returned from\n this function.\nSynonyms:\nbased.one\nbased one","url":"based.html#one"},{"doc":"based","title":"register","content":"pub fn register(\n  based_adapter: BasedAdapter(a, b, c),\n  callback: fn(DB) -&gt; c,\n) -&gt; c\n Expects a `with_connection` function and its first required argument. For a library\n implementing a `with_connection` function, the required argument will likely be its\n configuration data.\n In the case of `based/testing.with_connection`, the required argument is the expected\n return data.\nSynonyms:\nbased.register\nbased register","url":"based.html#register"},{"doc":"based","title":"string","content":"pub fn string(value: String) -&gt; Value\n Converts a string to a `Value` type\nSynonyms:\nbased.string\nbased string","url":"based.html#string"},{"doc":"based","title":"with_values","content":"pub fn with_values(query: Query, values: List(Value)) -&gt; Query\n Appends a list of values to the provided Query record.\nSynonyms:\nbased.with_values\nbased with_values","url":"based.html#with_values"},{"doc":"based","title":"based","content":"","url":"based.html"},{"doc":"based/testing","title":"Connection","content":"pub type Connection {\n  Connection(Subject(Message))\n}\n A mock connection\nConnection(Subject(Message))\n\n\nSynonyms:\nbased/testing.Connection\nbased/testing Connection","url":"based/testing.html#Connection"},{"doc":"based/testing","title":"Message","content":"pub type Message {\n  Shutdown\n  Insert(State)\n  Get(\n    reply_with: Subject(Result(List(Dynamic), BasedError)),\n    key: String,\n  )\n}\n\nShutdown\n\n\nInsert(State)\n\n\nGet(\n  reply_with: Subject(Result(List(Dynamic), BasedError)),\n  key: String,\n)\n\n\nSynonyms:\nbased/testing.Message\nbased/testing Message","url":"based/testing.html#Message"},{"doc":"based/testing","title":"State","content":"pub type State {\n  State(init: Dict(String, List(Dynamic)))\n}\n\nState(init: Dict(String, List(Dynamic)))\n\n\nSynonyms:\nbased/testing.State\nbased/testing State","url":"based/testing.html#State"},{"doc":"based/testing","title":"add","content":"pub fn add(\n  state: State,\n  key: String,\n  value: List(Dynamic),\n) -&gt; State\n\nSynonyms:\nbased/testing.add\nbased/testing add","url":"based/testing.html#add"},{"doc":"based/testing","title":"empty_returns_for","content":"pub fn empty_returns_for(queries: List(Query)) -&gt; State\n\nSynonyms:\nbased/testing.empty_returns_for\nbased/testing empty_returns_for","url":"based/testing.html#empty_returns_for"},{"doc":"based/testing","title":"insert","content":"pub fn insert(conn: Connection, state: State) -&gt; Connection\n\nSynonyms:\nbased/testing.insert\nbased/testing insert","url":"based/testing.html#insert"},{"doc":"based/testing","title":"mock_adapter","content":"pub fn mock_adapter(\n  state: State,\n) -&gt; BasedAdapter(State, Connection, a)\n\nSynonyms:\nbased/testing.mock_adapter\nbased/testing mock_adapter","url":"based/testing.html#mock_adapter"},{"doc":"based/testing","title":"mock_service","content":"pub fn mock_service(\n  query: Query,\n  conn: Connection,\n) -&gt; Result(List(Dynamic), BasedError)\n\nSynonyms:\nbased/testing.mock_service\nbased/testing mock_service","url":"based/testing.html#mock_service"},{"doc":"based/testing","title":"new_state","content":"pub fn new_state() -&gt; State\n\nSynonyms:\nbased/testing.new_state\nbased/testing new_state","url":"based/testing.html#new_state"},{"doc":"based/testing","title":"with_connection","content":"pub fn with_connection(\n  state: State,\n  callback: fn(Connection) -&gt; a,\n) -&gt; a\n For testing code without hitting a real database\nSynonyms:\nbased/testing.with_connection\nbased/testing with_connection","url":"based/testing.html#with_connection"},{"doc":"based/testing","title":"based/testing","content":"","url":"based/testing.html"}]);